mod lib_test;

use {
  reqwest::get,
  serde::Deserialize,
  std::{
    io::{
      Error,
      ErrorKind
    },
    sync::LazyLock
  },
  tokio::sync::RwLock
};

#[derive(Clone, Deserialize)]
pub struct TokenServiceApi {
  pub main:         String,
  pub youtube:      String,
  pub octokit:      String,
  pub postgres_uri: String,
  pub redis_uri:    String,
  pub wg_pms:       String,
  pub ilo_ip:       String,
  pub ilo_user:     String,
  pub ilo_pw:       String,
  pub topgg:        String
}

const TOKENSERVICE_BASE: &str = "http://ts.local.toast-data.center";

static GLOBAL_CACHE: LazyLock<RwLock<Option<TokenServiceApi>>> = LazyLock::new(|| RwLock::new(None));

pub struct TokenService {
  endpoint: String,
  server:   String
}

impl TokenService {
  pub fn new(endpoint: &str) -> Self {
    TokenService {
      endpoint: endpoint.to_owned(),
      server:   TOKENSERVICE_BASE.to_owned()
    }
  }

  pub async fn connect(&self) -> Result<TokenServiceApi, Box<dyn std::error::Error + Send + Sync>> {
    {
      let cache = GLOBAL_CACHE.read().await;
      if let Some(api) = &*cache {
        return Ok(api.clone())
      }
    }

    let response = match get(&format!("{}/{}", self.server, self.endpoint)).await {
      Ok(response) => response,
      Err(y) => return Err(Box::new(Error::new(ErrorKind::Other, format!("GET request failed: {y}"))))
    };

    if !response.status().is_success() {
      return Err(Box::new(Error::new(
        ErrorKind::Other,
        format!("Received an error response code from server: {}", response.status())
      )))
    };

    let api: TokenServiceApi = match serde_json::from_str(&response.text().await.unwrap()) {
      Ok(api) => api,
      Err(err) => {
        return Err(Box::new(Error::new(
          ErrorKind::Other,
          format!("Failed to deserialize response into TokenServiceApi: {err}")
        )))
      },
    };

    {
      let mut cache = GLOBAL_CACHE.write().await;
      *cache = Some(api.clone());
    }

    Ok(api)
  }
}
